<?php

class Errors_install_model extends N_Model {

    private $CI;
    private $module_version = 3;
    private $sub_version = 2;
    private $patch_version = 0;
    private $name = 'Errors';
    private $module_type = 'mandatory_product';
    private $keyword = 'errors';

    public function __construct() {
        parent::__construct();
        $this->CI = get_instance();
    }

    public function install() {
        $this->CI->load->helper('Modules');
        if (function_exists('get_module_by_keyword')) {
            $module = get_module_by_keyword($this->keyword);
            if ($module) {
                $module->module_version = $this->module_version;
                $module->sub_version = $this->sub_version;
                $module->patch_version = $this->patch_version;
            } else {
                $module = array(
                    'keyword' => $this->keyword,
                    'name' => $this->name,
                    'module_type' => $this->module_type,
                    'module_version' => $this->module_version,
                    'sub_version' => $this->sub_version,
                    'patch_version' => $this->patch_version,
                );
            }
            if (function_exists('save_module')) {
                save_module($module);
            }
        }
    }

    public function update($module_version = 0, $sub_version = 0, $patch_version) {
        switch ($sub_version) {
            case 1:
                $this->_install_languages('3_2_0');
                break;
        }
        $this->CI->load->helper('Modules');
        if (function_exists('get_module_by_keyword')) {
            $module = get_module_by_keyword($this->keyword);
            if ($module) {
                $module->module_version = $this->module_version;
                $module->sub_version = $this->sub_version;
                $module->patch_version = $this->patch_version;
            } else {
                $module = array(
                    'keyword' => $this->keyword,
                    'name' => $this->name,
                    'module_type' => $this->module_type,
                    'module_version' => $this->module_version,
                    'sub_version' => $this->sub_version,
                    'patch_version' => $this->patch_version,
                );
            }
            if (function_exists('save_module')) {
                save_module($module);
            }
        }
    }

    public function install_seo() {
        $seo = array();
        $this->CI->load->helper('Options');
        $orig_languages = false;
        if (function_exists('load_options')) {
            $orig_languages = load_options('languages');
        }
        if (!$orig_languages) {
            $orig_languages = array(
                'en' => 'english',
            );
        }
        $russian_exists = true;
        if (!$this->db->field_exists('russian', 'seo')) {
            $russian_exists = false;
        }
        if (file_exists(MODULESPATH . 'errors/install/seo.php'))
            @include_once(MODULESPATH . 'errors/install/seo.php');
        foreach ($seo as $value) {
            $this->db->delete('seo', array('url' => $value['url']));
            if (!$russian_exists) {
                unset($value['title']['russian']);
                unset($value['description']['russian']);
                unset($value['keywords']['russian']);
            }
            foreach ($orig_languages as $lang_id => $lang_value) {
                if (!isset($value['title'][$lang_value]) && isset($value['title']['english'])) {
                    $value['title'][$lang_value] = $value['title']['english'];
                }
                if (!isset($value['description'][$lang_value]) && isset($value['description']['english'])) {
                    $value['description'][$lang_value] = $value['description']['english'];
                }
                if (!isset($value['keywords'][$lang_value]) && isset($value['keywords']['english'])) {
                    $value['keywords'][$lang_value] = $value['keywords']['english'];
                }
            }
            $value['title'] = @serialize($value['title']);
            $value['description'] = @serialize($value['description']);
            $value['keywords'] = @serialize($value['keywords']);
            $this->db->insert('seo', $value);
        }
        $this->db->cache_delete_prefix('seo');
    }

    public function install_languages() {
        $languages = array();
        if (file_exists(MODULESPATH . 'errors/install/languages.php'))
            @include_once(MODULESPATH . 'errors/install/languages.php');
        foreach ($languages as $language) {
            $this->db->delete('languages', array('key' => $language['key'], 'module' => $language['module']));
            $language['default'] = 1;
            $this->db->insert('languages', $language);
        }
        $this->db->cache_delete_prefix('languages');
    }

    protected function _install_languages($version = '') {
        $languages = array();
        $this->CI->load->helper('Options');
        $orig_languages = false;
        if (function_exists('load_options')) {
            $orig_languages = load_options('languages');
        }
        if (!$orig_languages) {
            $orig_languages = array(
                'en' => 'english',
            );
        }
        if (file_exists(MODULESPATH . 'errors/install/languages_' . $version . '.php'))
            @include_once(MODULESPATH . 'errors/install/languages_' . $version . '.php');
        if (is_array($languages) && count($languages) > 0) {
            $russian_exists = true;
            if (!$this->db->field_exists('russian', 'languages')) {
                $russian_exists = false;
            }
            foreach ($languages as $language) {
                $this->db->delete('languages', array('key' => $language['key'], 'module' => $language['module']));
                $language['default'] = 1;
                if (!$russian_exists) {
                    unset($language['russian']);
                }
                foreach ($orig_languages as $lang_id => $lang_value) {
                    if (!isset($language[$lang_value])) {
                        $language[$lang_value] = $language['english'];
                    }
                }
                $this->db->insert('languages', $language);
            }
        }
        $this->db->cache_delete_prefix('languages');
    }

}
