<?php

class Modules_install_model extends N_Model {

    public $CI;
    private $module_version = 3;
    private $sub_version = 2;
    private $patch_version = 1;
    private $name = 'Modules';
    private $module_type = 'mandatory_product';
    private $keyword = 'modules';

    public function __construct() {
        parent::__construct();
        $this->CI = get_instance();
    }

    public function install() {
        $this->load->database();
        $this->load->dbforge();
        if (!$this->db->table_exists('modules')) {
            $fields = array(
                'id' => array(
                    'type' => 'INT',
                    'constraint' => '11',
                    'null' => FALSE,
                    'auto_increment' => TRUE,
                ),
                'keyword' => array(
                    'type' => 'VARCHAR',
                    'constraint' => '255',
                    'default' => '',
                    'null' => FALSE,
                ),
                'name' => array(
                    'type' => 'VARCHAR',
                    'constraint' => '255',
                    'default' => '',
                    'null' => FALSE,
                ),
                'description' => array(
                    'type' => 'TEXT',
                    'null' => TRUE,
                ),
                'module_type' => array(
                    'type' => 'VARCHAR',
                    'constraint' => '255',
                    'default' => '',
                    'null' => FALSE,
                ),
                'module_version' => array(
                    'type' => 'INT',
                    'constraint' => '11',
                    'default' => '0',
                    'null' => FALSE,
                ),
                'sub_version' => array(
                    'type' => 'INT',
                    'constraint' => '11',
                    'default' => '0',
                    'null' => FALSE,
                ),
                'patch_version' => array(
                    'type' => 'INT',
                    'constraint' => '11',
                    'default' => '0',
                    'null' => FALSE,
                ),
                'status' => array(
                    'type' => 'TINYINT',
                    'constraint' => '1',
                    'default' => '0',
                    'null' => FALSE,
                ),
                'date_created' => array(
                    'type' => 'INT',
                    'constraint' => '11',
                    'default' => '0',
                    'null' => FALSE,
                ),
                'date_modified' => array(
                    'type' => 'INT',
                    'constraint' => '11',
                    'default' => '0',
                    'null' => FALSE,
                ),
            );
            $this->create_table('modules', $fields, array('id'));
        }
        $this->CI->load->helper('Modules');
        if (function_exists('get_module_by_keyword')) {
            $module = get_module_by_keyword($this->keyword);
            if ($module) {
                $module->module_version = $this->module_version;
                $module->sub_version = $this->sub_version;
                $module->patch_version = $this->patch_version;
            } else {
                $module = array(
                    'keyword' => $this->keyword,
                    'name' => $this->name,
                    'module_type' => $this->module_type,
                    'module_version' => $this->module_version,
                    'sub_version' => $this->sub_version,
                    'patch_version' => $this->patch_version,
                );
            }
            if (function_exists('save_module')) {
                save_module($module);
            }
        }
        $this->db->cache_delete_prefix('modules');
    }

    public function update($module_version = 0, $sub_version = 0, $patch_version) {
        switch ($sub_version) {
            case 1:
                $fields = array(
                    'description' => array(
                        'type' => 'TEXT',
                        'null' => TRUE,
                    ),
                );
                $this->create_table('modules', $fields, null, true);
                $this->_install_languages('3_2_0');
            case 2:
                switch ($patch_version) {
                    case 0:
                        $this->_install_languages('3_2_1');
                }
                break;
        }
        $this->CI->load->helper('Modules');
        if (function_exists('get_module_by_keyword')) {
            $module = get_module_by_keyword($this->keyword);
            if ($module) {
                $module->module_version = $this->module_version;
                $module->sub_version = $this->sub_version;
                $module->patch_version = $this->patch_version;
            } else {
                $module = array(
                    'keyword' => $this->keyword,
                    'name' => $this->name,
                    'module_type' => $this->module_type,
                    'module_version' => $this->module_version,
                    'sub_version' => $this->sub_version,
                    'patch_version' => $this->patch_version,
                );
            }
            if (function_exists('save_module')) {
                save_module($module);
            }
        }
        $this->db->cache_delete_prefix('modules');
        $this->CI->load->helper('Modules');
        if (function_exists('get_modules_list')) {
            $modules = get_modules_list(null, null, null);
        } else {
            $modules = array();
        }
        if (is_array($modules) && count($modules) > 0) {
            foreach ($modules as $module) {
                if ($module != 'modules') {
                    $model_name = $module['keyword'] . '_install_model';
                    if (file_exists(MODULESPATH . $module['keyword'] . '/models/' . $model_name . '.php')) {
                        $this->CI->load->model($model_name);
                        if (method_exists($this->CI->$model_name, 'update'))
                            $this->CI->$model_name->update($module['module_version'], $module['sub_version'], $module['patch_version']);
                    }
                }
            }
        }
    }

    public function install_languages() {
        $languages = array();
        if (file_exists(MODULESPATH . 'modules/install/languages.php'))
            @include_once(MODULESPATH . 'modules/install/languages.php');
        foreach ($languages as $language) {
            $this->db->delete('languages', array('key' => $language['key'], 'module' => $language['module']));
            $language['default'] = 1;
            $this->db->insert('languages', $language);
        }
        $this->db->cache_delete_prefix('languages');
    }

    protected function _install_languages($version = '') {
        $languages = array();
        $this->CI->load->helper('Options');
        $orig_languages = false;
        if (function_exists('load_options')) {
            $orig_languages = load_options('languages');
        }
        if (!$orig_languages) {
            $orig_languages = array(
                'en' => 'english',
            );
        }
        if (file_exists(MODULESPATH . 'modules/install/languages_' . $version . '.php'))
            @include_once(MODULESPATH . 'modules/install/languages_' . $version . '.php');
        if (is_array($languages) && count($languages) > 0) {
            $russian_exists = true;
            if (!$this->db->field_exists('russian', 'languages')) {
                $russian_exists = false;
            }
            foreach ($languages as $language) {
                $this->db->delete('languages', array('key' => $language['key'], 'module' => $language['module']));
                $language['default'] = 1;
                if (!$russian_exists) {
                    unset($language['russian']);
                }
                foreach ($orig_languages as $lang_id => $lang_value) {
                    if (!isset($language[$lang_value])) {
                        $language[$lang_value] = $language['english'];
                    }
                }
                $this->db->insert('languages', $language);
            }
        }
        $this->db->cache_delete_prefix('languages');
    }

}
