<?php

(defined('BASEPATH')) OR exit('No direct script access allowed');

/* load the MX core module class */
require dirname(__FILE__) . '/Modules.php';

/**
 * Modular Extensions - HMVC
 *
 * Adapted from the CodeIgniter Core Classes
 * @link        http://codeigniter.com
 *
 * Description:
 * This library extends the CodeIgniter router class.
 *
 * Install this file as application/third_party/MX/Router.php
 *
 * @copyright   Copyright (c) 2011 Wiredesignz
 * @version     5.4
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 * */
class MX_Router extends CI_Router {

    private $module;

    public function fetch_module() {
        return $this->module;
    }

    public function _validate_request($segments) {

        if (count($segments) == 0)
            return $segments;

        /* locate module controller */
        if ($located = $this->locate($segments))
            return $located;

        /* use a default 404_override controller */
        if (isset($this->routes['404_override']) AND $this->routes['404_override']) {
            $segments = explode('/', $this->routes['404_override']);
            if ($located = $this->locate($segments))
                return $located;
        }

        /* no controller found */
        show_404();
    }

    /** Locate the controller * */
    public function locate($segments) {

        $this->module = '';
        $this->directory = '';
        $ext = $this->config->item('controller_suffix') . EXT;

        /* use module route if available */
        if (isset($segments[0]) AND $routes = Modules::parse_routes($segments[0], implode('/', $segments))) {
            $segments = $routes;
        }

        /* get the segments array elements */
        list($module, $directory, $controller) = array_pad($segments, 3, NULL);

        $this->lang_mode = FALSE;
        
        if (strlen($module) == 2){
            $this->lang_mode = $module;
            
            foreach($segments as $segment_id => $segment_value) {
                    if (isset($segments[$segment_id+1])) {
                        $segments[$segment_id] = $segments[$segment_id+1];
                    } else {
                        unset($segments[$segment_id]);
                    }
            }
            
            if (count($segments) > 0) {
                $module =  $segments[0];
            } else {
                $segments[0] = 'main';
                $module =  $segments[0];
            }

        }
        
        $admin_mode = FALSE;
        $cron_mode = FALSE;

        if (in_array($module, array('admin', 'cron'))) {
            switch($module) {
                case 'admin':
                    $admin_mode = TRUE;
                    break;
                case 'cron':
                    $cron_mode = TRUE;
                    break;
            }
            if (!isset($segments[1]) || $segments[1] == '') {
                $segments[1] = 'main';
                $segments[2] = 'index';
            }

            if (isset($segments[1]) && $segments[1] != '' && (!isset($segments[2]) || $segments[2] == '')) {
                $segments[2] = 'index';
            }
            if (is_file(APPPATH . 'modules/' . $segments[1] . '/controllers/' . $module . '_' . $segments[1] . '.php')) {
                $segments[0] = $module . '_' . $segments[1];
            } else {
                $segments[0] = count($segments) > 1 ? (($segments[1] = 'main') ? $module . '_main' : $module . '_' . $segments[1]) : '';
            }

            foreach($segments as $segment_id => $segment_value) {
                if ($segment_id > 0) {
                    if (isset($segments[$segment_id+1])) {
                        $segments[$segment_id] = $segments[$segment_id+1];
                    } else {
                        unset($segments[$segment_id]);
                    }
                }
            }
            
            $old_module_name = $module;
            
            $module = count($segments) > 0 ? $segments[0] : 'main';

            list($module, $directory, $controller) = array_pad($segments, 3, NULL);
            $module = str_replace($old_module_name . '_', '', $module);

            if (!is_file(APPPATH . 'modules/' . $module . '/controllers/' . $old_module_name . '_' . $module . '.php'))
                show_404();
        }

        /* check modules */
        foreach (Modules::$locations as $location => $offset) {
            /* module exists? */
            if (is_dir($source = $location . $module . '/controllers/')) {

                $this->module = $module;
                $this->directory = $offset . $module . '/controllers/';
                /* module sub-controller exists? */
                if ($directory AND is_file($source . $directory . $ext)) {
                    return array_slice($segments, $this->lang_mode ? 0 : 1);
                }
                
                /* module sub-directory exists? */
                if ($directory AND is_dir($source . $directory . '/')) {

                    $source = $source . $directory . '/';
                    $this->directory .= $directory . '/';

                    /* module sub-directory controller exists? */
                    if (is_file($source . $directory . $ext)) {
                        return array_slice($segments, $this->lang_mode ? 0 : 1);
                    }

                    /* module sub-directory sub-controller exists? */
                    if ($controller AND is_file($source . $controller . $ext)) {
                        return array_slice($segments, $this->lang_mode ? 1 : 2);
                    }
                }
                
                ksort($segments);
                
                /* module controller exists? */
                /*
                if (is_file($source . $module . $ext) || ($admin_mode && is_file($source . 'admin_' . $module . $ext)) || ($cron_mode && is_file($source . 'cron_' . $module . $ext))) {
                    if (preg_match('/admin/', $segments[0])) {
                        unset($segments[4]);
                        ksort($segments);
                    }
                    if (preg_match('/cron/', $segments[0])) {
                        unset($segments[4]);
                        ksort($segments);
                    }
                    return $segments;
                } 
                */
                return $segments;
            }
        }

        show_404();

        /* application controller exists? */
        if (is_file(APPPATH . 'controllers/' . $module . $ext)) {
            return $segments;
        }

        /* application sub-directory controller exists? */
        if ($directory AND is_file(APPPATH . 'controllers/' . $module . '/' . $directory . $ext)) {
            $this->directory = $module . '/';
            return array_slice($segments, 1);
        }

        /* application sub-directory default controller exists? */
        if (is_file(APPPATH . 'controllers/' . $module . '/' . $this->default_controller . $ext)) {
            $this->directory = $module . '/';
            return array($this->default_controller);
        }
    }

    public function set_class($class) {
        $this->class = $class . $this->config->item('controller_suffix');
    }

}
